/****************************************************************/
/* Idenity_Gen - This utility produces an identity martix.	*/
/* A square matrix having all ones down the main diagonal, and 	*/
/* zeros off diagonal.						*/
/*								*/
/* Command line options:					*/
/*	{file_name} - Output file name.				*/
/*	-n kk	- Create matrix of dimension kk.		*/
/*	-complex - Produce a complex matrix.			*/
/*	-scale xx - Main diagonal will have value xx 		*/
/*		instead of  1.0.				*/
/*	-sizeofmat matname - Create matrix of dimension		*/
/*		matching that of the one in file 'matname'.	*/
/*								*/
/* By default, this utility produces a real matrix of dimension	*/
/* 3x3, unless otherwise specified.				*/
/*								*/
/* File Types:							*/
/*      Out:  Real (default), or Complex (option)		*/
/*								*/
/* From:  www.xgraph.org/numutil				*/
/*	  xgraphadmin@xgraph.org				*/
/*								*/
/* Copyright (C) 2002 						*/
/* This program is free software; you can redistribute it	*/
/* and/or  modify it under the terms of the GNU General		*/
/* Public License as published by the Free Software		*/
/* Foundation, Version 2 of the GPL.  This program is		*/
/* distributed in the hope that it will be useful, but		*/
/* WITHOUT ANY WARRANTY; without even the implied warranty	*/
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	*/
/* See the GNU General Public License for more details.		*/
/* You should have received a copy of the GNU General		*/
/* Public License along with this program; if not, write to	*/
/* the  Free Software Foundation, Inc., 59 Temple Place,	*/
/* Suite 330,  Boston, MA 02111-1307. 				*/
/****************************************************************/

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <math.h>

#include "commonlibs.c"


int main ( int argc, char *argv[] )
{
 int i, cmat=0, m=3, k, row, col;
 float scale=1.0;
 FILE *outf;
 char *fname=0;
 struct file_kind *infile=0;

 i = 1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-complex")==0) cmat = 1; else
  if (strcmp(argv[i],"-n")==0)
   {
    i = i + 1;
    if (i==argc) { printf("ERROR Identity_Gen: Missing number after '-n'\n"); exit(1); }
    if (sscanf(argv[i],"%d",&m)!=1)
     { printf("ERROR Identity_Gen: Dimension '-n %s' not integer.\n",argv[i]); exit(1); }
   }
  else
  if (strcmp(argv[i],"-scale")==0)
   {
    i = i + 1;  
    if (i==argc) { printf("ERROR Identity_Gen: Missing value after '-scale'\n"); exit(1); }
    if (sscanf(argv[i],"%f",&scale)!=1)
     { printf("ERROR Identity_Gen: Scale value '%s' not float.\n",argv[i]); exit(1); }
   }
  else
  if (strcmp(argv[i],"-sizeof")==0)
   {
    i = i + 1;
    infile = file_open_read( argv[i] );
    if (infile->fpt == 0) { printf("ERROR Identity_Gen: Cannot open file '%s'\n",argv[i]); exit(1); }
    if (infile->dim1 != infile->dim2) { printf("ERROR Identity_Gen: Sizeof matrix not square.\n");  exit(1);}
    m = infile->dim1;
printf("Size is %d\n", infile->dim1 );
   }
  else
  fname = (char *)strdup(argv[i]);  
  i = i + 1;
 }

 if (m < 1)
  { printf("Dimension cannot be less than one.\n"); exit(1); }

 if (fname==0)
   fname = strdup( "identity_matrix.dat" );
 printf("	(Identity_Gen writing matrix file %s)\n", fname );

 outf = fopen(fname,"w");
 if (outf==0)
  { printf("ERROR rand_gen: Cannot open '%s' for writing.\n"); exit(0); }

 if (cmat)
  {
   printf("Producing complex identity matrix to file '%s'.\n", fname);
   fprintf(outf,"! <Type> Complex Matrix[%d,%d] </Type>\n", m, m );
  }
 else
  {
   printf("Producing real identity matrix to file '%s'.\n", fname);
   fprintf(outf,"! <Type> Real Matrix[%d,%d] </Type>\n", m, m );
  }

 if (cmat)
  {
   for (row=0; row < m; row++)
    for (col=0; col < m; col++)
     if (row != col)
      fprintf(outf,"%d %d	0.0	0.0\n", row, col );
     else
      fprintf(outf,"%d %d	%g	0.0\n", row, col, scale );
   }  
  else
  {
   for (row=0; row < m; row++)
    for (col=0; col < m; col++)
     if (row != col)
      fprintf(outf,"%d %d	0.0\n", row, col );
     else
      fprintf(outf,"%d %d	%g\n", row, col, scale );
   }  
 fclose(outf);
}
